# Basic OBJ file viewer. needs objloader from:
#  http://www.pygame.org/wiki/OBJFileLoader
# LMB + move: rotate
# RMB + move: pan
# Scroll wheel: zoom in/out
import sys, os, pygame, operator
from pygame.locals import *
from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *

# init before we get any further
pygame.init()
glutInit(sys.argv[1:])
viewport = (1024, 768)
screen = pygame.display.set_mode(viewport, OPENGL | DOUBLEBUF) # | FULLSCREEN)

from map import *
import ui, objects, textures, rabbit, farmer, fonts
import euclid, pyglyph

class Game(ui.UserInterface):
    def __init__(self, viewport):
        self.viewport = viewport
        hx = self.viewport[0]/2
        hy = self.viewport[1]/2
        pygame.mouse.set_visible(False)

        glLightfv(GL_LIGHT0, GL_POSITION,  (100, 200, 100, 0.0))
        glLightfv(GL_LIGHT0, GL_AMBIENT, (0.2, 0.2, 0.2, 1.0))
        glLightfv(GL_LIGHT0, GL_DIFFUSE, (0.5, 0.5, 0.5, 1.0))
        glEnable(GL_LIGHT0)
        glEnable(GL_LIGHTING)
        glEnable(GL_COLOR_MATERIAL)
        glEnable(GL_DEPTH_TEST)
        glShadeModel(GL_SMOOTH)

        # now we're initialised, load everything up
        objects.load()
        textures.load()
        rabbit.Rabbit.load()
        farmer.Farmer.load()

    def menu(self):
        easy = pyglyph.layout_text('(E)asy', font=fonts.sans40)
        hard = pyglyph.layout_text('(H)ard', font=fonts.sans40)
        
        press = pyglyph.layout_html('''
            <font face="bitstream vera sans" size="20">Press
            E or H to start
            <br>
            or escape to quit</font>''', font_factory=fonts.fonts)

        level = 1

        clock = pygame.time.Clock()
        timeout = 0
        start = False
        while 1:
            ts = clock.tick(20)
            timeout += ts
            for e in pygame.event.get():
                if e.type == QUIT: sys.exit()
                if not hasattr(e, 'key'): continue
                if not e.type == KEYDOWN: continue
                if e.key in (K_e, K_h):
                    if e.key == K_e: difficulty = 1
                    else: difficulty = 2
                    if timeout > 500:
                        start = True
                        level = 1
                        timeout = 0
                if e.key == K_ESCAPE: return

            self.twod_setup(colour=(0.52, .78, .47))
            x, y = self.viewport

            glPushMatrix()
            #glEnable(GL_BLEND)
            #glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
            tex = textures.textures['bouncy-title']
            glTranslate(x/2 - tex.width/2, y-tex.height, 0)
            tex.render()
            #glDisable(GL_BLEND)
            glPopMatrix()

            pyglyph.begin()
            easy.draw(pos=(x/2, y - tex.height),
                anchor=(pyglyph.Align.center, pyglyph.Align.top))
            hard.draw(pos=(x/2, y - tex.height - 50),
                anchor=(pyglyph.Align.center, pyglyph.Align.top))
            press.draw(pos=(x/2, y - tex.height - 150),
                anchor=(pyglyph.Align.center, pyglyph.Align.top))
            pyglyph.end()

            if start:
                if timeout < 300:
                    self.fade('out', timeout, 300)
                else:
                    start = False
                    while 1:
                        if not os.path.exists('data/level%s.csv'%level):
                            self.ok_dialog('You finished the game!')
                            level = 1
                            break
                        elif level > 1:
                            self.ok_dialog('Level complete!')
                        map = Map(self.viewport, 'data/level%s.csv'%level)
                        if not map.play(difficulty): break
                        level += 1
                    timeout = 0
            elif timeout < 300:
                self.fade('in', timeout, 300)

            pygame.display.flip()

if __name__ == '__main__':
    game = Game(viewport)
    game.menu()
#    map = Map(viewport, 'data/test_level.csv')
#    map = Map(viewport, 'data/level2.csv')
#    map.play(1)

